#!/bin/bash

#
#  usage : clpcloud.sh <sub-command> <option>
# 
#  sub-command :
#         get-cloud-info
#         aws-put-metric-data <namespace> <metric-data>...
#         aws-publish-topic <topic-arn> <message>
#

export PATH=$PATH:/usr/local/bin

CURL_TIMEOUT=5
AWS_API_VERSION=2019-10-01
AWS_TOKEN_TTL=60
AWS_CLI_TIMEOUT=10
AZURE_API_VERSION=2019-11-01

#==================================
# AWS環境情報取得
#==================================
get_aws_info()
{
    token=$(curl -is -m "${CURL_TIMEOUT}" -X PUT "http://169.254.169.254/latest/api/token" \
            -H "X-aws-ec2-metadata-token-ttl-seconds: ${AWS_TOKEN_TTL}" \
            | tr -d "\r" | (
                read -r _ statcode _
                test "${statcode:=0}" -eq 200 || exit 1
                while read -r line
                do
                    test "${line}" = "Server: EC2ws" && ok=1
                done
                echo "${line}"
                test "${ok}"
            ))

    if [ "$?" -ne 0 ]; then
        return
    fi

    echo "PFTYPE=AWS"

    # ----INSTANCE-ID----
    instance_id=$(curl -sS "http://169.254.169.254/${AWS_API_VERSION}/meta-data/instance-id" \
        -H "X-aws-ec2-metadata-token: ${token}" -m "${CURL_TIMEOUT}") \
        || echo "Could not get [instance-id]" >&2
    [ -n "${instance_id}" ] && echo "INSTANCE_ID=${instance_id}"

    interfaces="http://169.254.169.254/${AWS_API_VERSION}/meta-data/network/interfaces"
    macs=$(curl -sS "${interfaces}/macs" -H "X-aws-ec2-metadata-token: ${token}" -m "${CURL_TIMEOUT}")

    if [ "$?" -ne 0 ]; then
        echo "Could not get [macs]" >&2
        exit 1
    fi

    # ----VPC-ID----
    for mac in ${macs}
    do
        vpc_id=$(curl -sS "${interfaces}/macs/${mac}vpc-id" \
            -H "X-aws-ec2-metadata-token: ${token}" -m "${CURL_TIMEOUT}") \
            || echo "Could not get [vpc-id]" >&2
        [ -n "${vpc_id}" ] && echo "VPC_ID=${vpc_id}"
    done

    # ----ENI-ID----
    for mac in ${macs}
    do
        eni_id=$(curl -sS "${interfaces}/macs/${mac}interface-id" \
            -H "X-aws-ec2-metadata-token: ${token}" -m "${CURL_TIMEOUT}") \
            || echo "Could not get [eni-id]" >&2
        [ -n "${eni_id}" ] && echo "ENI_ID=${eni_id}"
    done

#    # ----EIP-ALLOCATION-ID----
#    eip_allocation_ids=$(aws ec2 describe-addresses --output text \
#        --cli-read-timeout "${AWS_CLI_TIMEOUT}" \
#        --cli-connect-timeout "${AWS_CLI_TIMEOUT}" \
#        --query 'Addresses[].AllocationId') \
#        || echo "Could not get [eip-allocation-id]" >&2
#
#    for key in ${eip_allocation_ids}
#    do
#        echo "EIP_ALLOCATION_ID=${key}"
#    done
#
#    # ----HOSTED-ZONE-ID----
#    hosted_zone_ids=$(aws route53 list-hosted-zones --output text \
#        --cli-read-timeout "${AWS_CLI_TIMEOUT}" \
#        --cli-connect-timeout "${AWS_CLI_TIMEOUT}" \
#        --query 'HostedZones[].Id') \
#        || echo "Could not get [hosted-zone-id]" >&2
#
#    for key in ${hosted_zone_ids}
#    do
#        echo "HOSTED_ZONE_ID=${key##*/}"
#    done

    # ----AZ-NAME----
    az_name=$(curl -sS "http://169.254.169.254/${AWS_API_VERSION}/meta-data/placement/availability-zone" \
        -H "X-aws-ec2-metadata-token: ${token}" -m "${CURL_TIMEOUT}") \
        || echo "Could not get [availability-zone]" >&2
    [ -n "${az_name}" ] && echo "AZ_NAME=${az_name}"

    exit 0
}


#==================================
# Azure環境情報取得
#==================================
get_azure_info()
{
    curl -is "http://169.254.169.254/metadata/instance?api-version=${AZURE_API_VERSION}&format=text" \
        -H "Metadata:true" -m "${CURL_TIMEOUT}" \
        | tr -d "\r" | (
            read -r _ statcode _
            [ "${statcode:=0}" -eq 200 ] && grep "^Server: IMDS" >/dev/null
        )

    if [ "$?" -ne 0 ]; then
        return
    fi

    echo "PFTYPE=Azure"
    exit 0
}


#==================================
# GCP環境情報取得
#==================================
get_gcp_info()
{
    curl -is "http://169.254.169.254/computeMetadata/v1/" \
        -H "Metadata-Flavor: Google" -m "${CURL_TIMEOUT}" \
        | tr -d "\r" | (
            read -r _ statcode _
            [ "${statcode:=0}" -eq 200 ] && grep "^Metadata-Flavor: Google" >/dev/null
        )

    if [ "$?" -ne 0 ]; then
        return
    fi

    echo "PFTYPE=GCP"
    exit 0
}


#==================================
# OCI環境情報取得
#==================================
get_oci_info()
{
    curl -Is "http://169.254.169.254/opc/v1/instance/" -m "${CURL_TIMEOUT}" \
        | tr -d "\r" | (
            read -r _ statcode _
            [ "${statcode:=0}" -eq 200 ]
        )

    if [ "$?" -ne 0 ]; then
        return
    fi

    echo "PFTYPE=OCI"
    exit 0
}


# ---------------------------- 処理開始 ----------------------------
sub_command="$1"

#==================================
# クラウド環境情報取得
#==================================
if [ "${sub_command}" = "get-cloud-info" ]; then

    curl -IsS -m "${CURL_TIMEOUT}" "http://169.254.169.254/" > /dev/null
 
    if [ "$?" -ne 0 ]; then
        echo "PFTYPE=UNKNOWN"
        exit 0
    fi

    get_aws_info
    get_azure_info
    get_gcp_info
    get_oci_info

    echo "PFTYPE=UNKNOWN"


#==================================
# Amazon Cloudwatchメトリクス送信
#==================================
elif [ "${sub_command}" = "aws-put-metric-data" ]; then

    if [ "$#" -lt 3 ]; then
        echo "Invalid Option!!" >&2
        exit 1
    fi

    namespace="$2"
    metric_data=("${@:3}")

    aws cloudwatch put-metric-data --namespace "${namespace}" --metric-data "${metric_data[@]}"

    if [ "$?" -ne 0 ]; then
        echo "Command Error. [aws cloudwatch put-metric-data]" >&2
        exit 1
    fi

#==================================
# Amazon SNSトピック発行
#==================================
elif [ "${sub_command}" = "aws-publish-topic" ]; then

    if [ "$#" -ne 3 ]; then
        echo "Invalid Option!!" >&2
        exit 1
    fi

    topic_arn="$2"; message="$3"
    aws sns publish --output text --topic-arn "${topic_arn}" --message "${message}"

    if [ "$?" -ne 0 ]; then
        echo "Command Error. [aws sns publish]" >&2
        exit 1
    fi


else
    echo "Invalid Sub Command!!" >&2
    exit 1

fi



exit 0
